#include "logicmain.h"
#include "src/frame/logger.h"
#include "src/frame/configuration.h"
#include "src/po/translation.h"
#include "src/model/monitorcdrom.h"
#include "src/data/datacdrom.h"
#include "src/model/menufactory.h"
#include "src/view/modaldialog.h"
#include "src/view/filefilterrules.h"

#include <unistd.h>
#include <pwd.h>

#include <QStandardPaths>
#include <QFileInfo>

LogicMain *LogicMain::m_oInstance = nullptr;
QMutex     LogicMain::m_oMutex;

LogicMain::LogicMain(QObject *parent) :
    QObject(parent),
    m_iBusness(-1),
    m_oTheme(nullptr)
{
    QFileInfo f;
    struct passwd *pwd;


    connect(this, SIGNAL(info(QString)), LOGGER, SLOT(info(QString)));
    connect(this, SIGNAL(warn(QString)), LOGGER, SLOT(warn(QString)));
    connect(this, SIGNAL(error(QString)), LOGGER, SLOT(error(QString)));
    connect(this, SIGNAL(debug(QString)), LOGGER, SLOT(debug(QString)));
    LOGGER->registerName(this, _("MAIN"));

    m_oCDROMS.clear();

    if (QGSettings::isSchemaInstalled("org.ukui.style"))
    {
        m_oTheme = new QGSettings("org.ukui.style");
        m_oCurrentIcon = m_oTheme->get("iconThemeName").toString();
        connect(m_oTheme, &QGSettings::changed, [=](){
            if (m_oCurrentIcon != m_oTheme->get("iconThemeName").toString())
            {
                m_oCurrentIcon = m_oTheme->get("iconThemeName").toString();
                emit iconChange(QIcon::fromTheme("burner"));
            }
        });
    }

    pwd = getpwuid(getuid());

    msg = QDBusMessage::createMethodCall( QString("com.kylinUserGuide.hotel_%1")
                                          .arg(pwd->pw_gid),"/", "com.guide.hotel",
                                          "showGuide");
    f.setFile("/usr/share/kylin-user-guide/data/guide/kylin-burner");
    msg << "kylin-burner";
    msg.setDelayedReply(false);
#if 0
    else
    {
        f.setFile("/usr/share/kylin-user-guide/data/guide/burner");
        if (f.exists()) msg << "kylin-burner";
        else
        {
            error(_("Cannot find help document."));
            return;
        }
    }
#endif

    connect(MONITOR, SIGNAL(addCDROM(QString)), this, SLOT(createCDROM(QString)));
    connect(MONITOR, SIGNAL(changeCDROM(QString)), this, SLOT(changeCDROM(QString)));
    connect(MONITOR, SIGNAL(removeCDROM(QString)), this, SLOT(removeCDROM(QString)));
    debug(_("Connect MONITOR signal addCDROM to createCDROM, for creating new"
            " cdrom information."));

    connect(MENU, SIGNAL(burnSetting(bool)), this, SLOT(slotsMenuBurnSetting()));    
    connect(this, SIGNAL(cdrom(void*)), MENU, SLOT(newCDROM(void*)));
    connect(MENU, SIGNAL(dataSetting(bool)), this, SLOT(slotsDataSetting()));
    connect(MENU, SIGNAL(diskCheck(bool)), this, SLOT(slotsDiskCheck()));
    connect(MENU, SIGNAL(help(bool)), this, SLOT(slotsHelp()));
    connect(this, SIGNAL(deleteCDROM(void*)), MENU, SLOT(takeCDROM(void*)));
}

LogicMain *LogicMain::instance()
{
    if (nullptr == m_oInstance)
    {
        QMutexLocker locker(&m_oMutex);

        if (nullptr == m_oInstance)
        {
            m_oInstance = new LogicMain();
        }
    }
    return m_oInstance;
}

void LogicMain::initISO()
{
    DataCDROM *c;
    QString path;

    path = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
    createCDROM(CFG->get("CreateISO", "DefaultPath", path).toString());
    path = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
    path = CFG->get("CopyImage", "DefaultPath", path).toString();
    if (!path.endsWith("/")) path.append("/");

    c = new DataCDROM(path, this);
    createCopyImage(c);
}

QString LogicMain::theme()
{
    QString ret;

    ret = m_oTheme->get("styleName").toString();
    if (ret == "ukui-dark") ret = "ukui-black";
    return ret;
}

QString LogicMain::icon()
{
    return m_oTheme->get("iconThemeName").toString();
}

void LogicMain::createCDROM(QString blockName)
{
    DataCDROM *cdrom;
    if (nullptr == sender())
    {
        blockName.append("/");
        blockName.append(CFG->get("CreateISO", "VolumnID", _("KylinBurner")).toString());
        blockName.append(".iso");
    }
    debug(_("Creating new cdrom named %1").arg(blockName));
    if (!MODAL->isHidden()) MODAL->accept();
    cdrom = static_cast<DataCDROM *>(m_oCDROMS.value(blockName));
    if (cdrom)
    {
        error(_("CDROM %1 already loaded.").arg(cdrom->display()));
        return;
    }
    else
    {
        cdrom = new DataCDROM(blockName, this);
    }
    m_oCDROMS.insert(blockName, cdrom);
    if (cdrom->isCDROM())
    {
        connect(cdrom, SIGNAL(empty()),this, SLOT(slotsEmptyMedium()));
        connect(cdrom, SIGNAL(appendale()), this, SLOT(slotsAppendableMedium()));
        connect(cdrom, SIGNAL(finished()), this, SLOT(slotsFinishedMedium()));
        cdrom->update();
    }
    else
    {
        emit createISO(cdrom);
    }
    emit this->cdrom(cdrom);
}

void LogicMain::changeCDROM(QString blockName)
{
    DataCDROM *cdrom;
    debug(_("Change CDROM, block name is %1").arg(blockName));
    QThread::sleep(1);

    cdrom = static_cast<DataCDROM *>(m_oCDROMS.value(blockName));
    if (nullptr == cdrom)
    {
        // createCDROM(blockName);
        return;
    }
    if (!MODAL->isHidden() && !cdrom->isEjected()) MODAL->close();
    debug(_("Change CDROM %1").arg(cdrom->display()));
    noMedium(cdrom);
    cdrom->update();
    if (cdrom->mediumStatus() < 0)
    {
        debug(_("There is no medium in CDROM %1").arg(cdrom->display()));
        noMedium(cdrom);
        return;
    }
}

void LogicMain::removeCDROM(QString block)
{
    DataCDROM *cdrom;

    cdrom = static_cast<DataCDROM *>(m_oCDROMS.value(block));
    if (!MODAL->isHidden()) MODAL->accept();
    if (nullptr == cdrom) return;
    debug(_("Remove CDROM %1").arg(cdrom->display()));
    m_oCDROMS.remove(block);
    cdrom->release();
    noMedium(cdrom); // 首先移除数据
    deleteCDROM(cdrom); // 移除界面展示中遗留的光驱信息。
}

void LogicMain::slotsEmptyMedium()
{
    DataCDROM *cdrom;

    cdrom = qobject_cast<DataCDROM *>(sender());

    debug(_("Get empty medium in CDROM %1").arg(cdrom->display()));
    debug(_("BurnerName : %1 DiskName : %2").arg(cdrom->display()).arg(cdrom->mDisplay()));
    emptyMedium(sender());
}

void LogicMain::slotsAppendableMedium()
{
    DataCDROM *cdrom;

    cdrom = qobject_cast<DataCDROM *>(sender());

    cdrom->use();
    debug(_("Get appendable medium in CDROM %1").arg(cdrom->display()));
    debug(_("BurnerName : %1 DiskName : %2").arg(cdrom->display()).arg(cdrom->mDisplay()));
    emit appendableMedium(sender());
    cdrom->release();
}

void LogicMain::slotsFinishedMedium()
{
    DataCDROM *cdrom;

    cdrom = qobject_cast<DataCDROM *>(sender());

    debug(_("Get finished medium in CDROM %1").arg(cdrom->display()));
    debug(_("BurnerName : %1 DiskName : %2").arg(cdrom->display()).arg(cdrom->mDisplay()));
    emit finishedMedium(sender());
}

void LogicMain::slotsMenuBurnSetting()
{
    if (m_iBusness < CreateISO || m_iBusness > GetISO) return;
    if (m_iBusness < BurnImage) burnDataSetting();
    else if (m_iBusness == BurnImage) burnImageSetting();
    else burnCopySetting();
}

void LogicMain::slotsSetBusness(int bus)
{
    if (bus < CreateISO || bus > GetISO)
    {
        error(_("Undefined busness."));
        emit menu(GETMENU(bus));
        return;
    }
    m_iBusness = bus;
    emit menu(GETMENU(bus));
    debug(_("Change to do busness %1").arg(m_iBusness));
}

void LogicMain::slotsDoBusness()
{
    switch (m_iBusness)
    {
    case CreateISO:
        debug(_("Prepare to create iso file."));
        emit createISO();
        break;
    case BurnData:
        debug(_("Prepare to burn data."));
        emit burnData();
        break;
    case AddBurn:
        debug(_("Prepare to append burn."));
        emit addBurn();
        break;
    case BurnImage:
        debug(_("Prepare to burn image."));
        emit burnImage();
        break;
    case CopyData:
        debug(_("Prepare to copy disk data."));
        emit copyData();
        break;
    case GetISO:
        debug(_("Prepare to get iso from disk data."));
        emit getISO();
        break;
    default:
        error(_("Invalid busness [%1]").arg(m_iBusness));
        break;
    }
}

void LogicMain::slotsDataSetting()
{
    DATASETTING;
    MODAL->show();
}

void LogicMain::slotsDiskCheck()
{
    DISKCHECK;
    MODAL->show();
}

void LogicMain::slotsHelp()
{
    QDBusConnection::sessionBus().call(msg);
}

LogicMain::~LogicMain()
{}
