#include "processburnimage.h"
#include "src/frame/configuration.h"
#include "src/frame/logger.h"
#include "src/po/translation.h"
#include "src/data/datacdrom.h"
#include "src/view/modaldialog.h"

#include <QFileInfo>
#include <QDebug>

ProcessBurnImage::ProcessBurnImage(QString image, void *cdrom, QObject *parent) :
    ProcessBasic(parent),
    m_oImage(image),
    m_oCDROM(cdrom),
    m_iProgress(0)
{
    LOGGER->registerName(this, _("PROCESS_BI"));
    title(_("Burning Image"));
    m_oProcessName = _("Burn Image");
    setRange(0, 100);
    setValue(0);
    connect(this, SIGNAL(eject()), static_cast<DataCDROM *>(m_oCDROM), SLOT(eject()));
}

bool ProcessBurnImage::prepare()
{
    QFileInfo f;
    DataCDROM *cdrom;
    info(_("Step.%1: preparing...").arg("prepare"));
    m_oTimer->setInterval(1000);
    setValue(0, m_oIcon, _("Step.%1: preparing...").arg("prepare"));
    m_oTimer->start();

    PROGRESS;
    MODAL->show();
    connect(this, SIGNAL(cancel(bool)), MODAL, SLOT(cancelEnable(bool)));

    f.setFile(m_oImage);
    if (!f.exists())
    {
        error(_("Image file %1 already been removed.").arg(m_oImage));
        setValue(++m_iProgress, QIcon::fromTheme("dialog-error-symbolic"),
                 _("Image file %1 already been removed.").arg(m_oImage));
        return false;
    }
    info(_("Checked image already exists."));
    setValue(++m_iProgress, m_oIcon, _("Checked image already exists."));

    cdrom = (DataCDROM *)getCDROM();
    if (nullptr == cdrom) return false;

    if (cdrom->mediumType().startsWith("CD"))
    {
        f.setFile("/bin/cdrecord");
        if (!f.exists())
        {
            error(_("Cannot find program <%1>.").arg(f.fileName()));
            setValue(++m_iProgress, QIcon::fromTheme("dialog-error-symbolic"),
                     _("Cannot find program <%1>.").arg(f.fileName()));
            return false;
        }
        if (!f.isExecutable())
        {
            error(_("Program %1 is not executable file.").arg(f.fileName()));
            setValue(++m_iProgress, QIcon::fromTheme("dialog-error-symbolic"),
                     _("Program %1 is not executable file.").arg(f.fileName()));
            return false;
        }
        m_oProcess->setProgram(f.fileName());
        m_oArguments << "-v";
        m_oArguments << QString("dev=%1").arg(cdrom->device());
        m_oArguments << "driveropts=burnfree";
    }
    else if (cdrom->mediumType().startsWith("DVD"))
    {
        f.setFile("/bin/growisofs");
        if (!f.exists())
        {
            error(_("Cannot find program <%1>.").arg(f.fileName()));
            setValue(++m_iProgress, QIcon::fromTheme("dialog-error-symbolic"),
                     _("Cannot find program <%1>.").arg(f.fileName()));
            return false;
        }
        if (!f.isExecutable())
        {
            error(_("Program %1 is not executable file.").arg(f.fileName()));
            setValue(++m_iProgress, QIcon::fromTheme("dialog-error-symbolic"),
                     _("Program %1 is not executable file.").arg(f.fileName()));
            return false;
        }
        m_oProcess->setProgram(f.fileName());
        m_oArguments << "-dvd-compat";
        m_oArguments << "-Z";
        m_oArguments << QString("%1=%2").arg(cdrom->device()).arg(m_oImage);
    }
    else
    {
        error(_("Donot suppot medium type %1 in CDROM.").arg(cdrom->mediumType()));
        setValue(++m_iProgress, QIcon::fromTheme("dialog-error-symbolic"),
                 _("Donot suppot medium type %1 in CDROM.").arg(cdrom->mediumType()));
        return false;
    }
    info(_("Checking program %1 can be supported.").arg(m_oProcess->program()));
    setValue(++m_iProgress, m_oIcon, _("Checking program %1 can be supported.").arg(m_oProcess->program()));
    return m_oKeep;
}

bool ProcessBurnImage::config()
{
    int speed;
    DataCDROM *cdrom;

    debug(_("Step.%1: getting...").arg("config"));
    setValue(++m_iProgress, m_oIcon, _("Step.%1: getting...").arg("config"));
    cdrom = (DataCDROM *)getCDROM();
    if (nullptr == cdrom) return false;

    speed = cdrom->settingSpeed();
    if (0 == speed)
    {
        warn(_("Speed config to 0, means auto."));
        addValue(++m_iProgress, QIcon::fromTheme("dialog-warning-symbolic"), _("Speed config to 0, means auto."));
    }
    if (cdrom->mediumType().startsWith("CD"))
    {
        if (speed) m_oArguments << QString("speed=%1").arg(speed);
        m_oArguments << "-eject";
        m_oArguments << m_oImage;
    }
    else
    {
        if (speed) m_oArguments << QString("-speed=%1").arg(speed);
    }
    info(_("Set burn speed %1").arg(speed ? QString("%1x").arg(speed) : "auto"));
    setValue(++m_iProgress, m_oIcon, _("Set burn speed %1").arg(speed ? QString("%1x").arg(speed) : "auto"));
    return m_oKeep;
}

void ProcessBurnImage::slotsStart()
{
    info(_("Started process %1").arg(m_oProcess->program()));
    setValue(++m_iProgress, m_oIcon, _("Started process %1").arg(m_oProcess->program()));
}

void ProcessBurnImage::slotsOutput()
{
    int v = m_iProgress;
    qreal si;
    qreal t;
    QString output;
    QStringList s;
    QStringList ss;

    output = m_oProcess->readAllStandardOutput();
    if (output.contains("Last chance"))
    {
        if (!output.contains("Operation starts."))
        {
            s = output.split(" ");
            s.takeLast();
            setValue(++v, m_oIcon, _("Start left %1 seconds.").arg(s.last()));
        }
        setValue(++v, m_oIcon, _("Started burn image, cannot stop."));
    }
    if (output.contains("Performing OPC"))
    {
        setValue(++v, m_oIcon, _("Performing OPC"));
        cancel(false);
    }
    if (output.startsWith("\rTrack"))
    {
        s = output.split(" ");
        for (int i = 0; i < s.size(); ++i)
        {
            if (s.at(i).isEmpty())
            {
                s.takeAt(i);
                --i;
            }
        }
        si = s.at(2).toDouble();
        t = s.at(4).toDouble();
        if (s.size() > 5)
        {
            setValue(v + si / t * 90.0, m_oIcon,
                     _("Writing track %1 speed %2 %3 of %4 %5 written.")
                     .arg(s.at(1)).arg(s.last()).arg(s.at(2)).arg(s.at(4)).arg(s.at(5)));
        }
    }
    if (output.contains("Fixating"))
    {
        setValue(99, m_oIcon, _("Fixating...(need waiting a moment.)"));
    }
    if (!output.isEmpty())
    {
        debug(_("%1 output : %2").arg(m_oProcess->program()).arg(output));
        if (m_oProcess->program().contains("growisofs"))
        {
            if (output.contains("UBU"))
            {
                s = output.split(")");
                ss = s.last().split("x,");
                ss = ss.first().split("@");
                m_fSpeed = ss.last().toDouble();
                s = s.first().split("(");
                t = s.last().split("%").first().toDouble();
                v = m_iProgress + t;
                if (v > 99) v = 99;
                if (m_fSpeed > 0.00)
                    setValue(v, m_oIcon, _("Burning image by speed %1x").arg(m_fSpeed));
            }
        }
    }
    if (v) m_iProgress = v;
}

void ProcessBurnImage::slotsError()
{
    QString error;

    error = m_oProcess->readAllStandardError();
    debug(_("%1 error : %2").arg(m_oProcess->program()).arg(error));
    qreal v;
    int basic;
    QString output;
    QStringList ss;

    output = m_oProcess->readAllStandardError();
    if (output.contains("-["))
        m_oKeep = false;
    if (output.contains("OPC failed."))
        m_oKeep = false;
    if (output.contains("Input/output error"))
        m_oKeep = false;
    basic = 99 - m_iProgress;
    cancel(false);

    if (output.contains("done, estimate"))
    {
        output = output.left(output.indexOf('%'));
        v = output.toDouble();
        v = (v / 100.00) * basic;
        m_iProgress += v;
        if (m_iProgress > 99) m_iProgress = 99;
        if (m_fSpeed > 0)
            setValue(m_iProgress, m_oIcon, _("Burning by speed %1x").arg(m_fSpeed));
    }
    if (output.contains("Current Write Speed"))
    {
        ss = output.split(" ");
        ss = ss.last().split("x");
        m_fSpeed = ss.first().toDouble();
    }

    if (output.contains("flushing cache"))
    {
        setValue(99, m_oIcon, _("flushing cache"));
    }
    if (output.contains("closing track"))
    {
        setValue(99, m_oIcon, _("closing track"));
    }
    if (output.contains("closing session"))
    {
        setValue(99, m_oIcon, _("closing session"));
    }
}

void ProcessBurnImage::slotsFinished(int code, QProcess::ExitStatus status)
{
    //DataCDROM *cdrom;

    if ("cdrecord" == m_oProcess->program())
    {
        if (code)
        {
            error(_("Error when burn image. %1").arg(m_oProcess->errorString()));
            m_oKeep = false;
        }
        else
        {
            info("burn image success.");
            m_oKeep = true;
            success(_("BurnImage"));
        }
    }
    else
    {
        if (code) m_oKeep = false;
    }
    //cdrom = (DataCDROM *)getCDROM();
    //cdrom->eject();

    debug(_("%1 finished : exit code : %2 status : %3").arg(m_oProcess->program()).arg(code).arg(status));
}

void *ProcessBurnImage::getCDROM()
{
    DataCDROM *cdrom;

    cdrom = (DataCDROM *)(m_oCDROM);
    if (nullptr == cdrom)
    {
        error(_("CDROM doesnot exists any more."));        
        addValue(0, QIcon::fromTheme("dialog-error-symbolic"), _("CDROM doesnot exists any more."));
        return nullptr;
    }
    if (!cdrom->isCDROM())
    {
        error(_("CDROM doesnot a real physical cdrom."));
        addValue(0, QIcon::fromTheme("dialog-error-symbolic"), _("CDROM doesnot a real physical cdrom."));
        return nullptr;
    }
    if (DataMedium::MEDIUM_EMPTY != cdrom->mediumStatus())
    {
        error(_("Medium in CDROM is not empty any more."));
        addValue(0, QIcon::fromTheme("dialog-error-symbolic"), _("Medium in CDROM is not empty any more."));
        return nullptr;
    }
    return cdrom;
}

ProcessBurnImage::~ProcessBurnImage()
{}
