#include "mainwindow.h"
#include "src/view/ui_mainwindow.h"
#include "src/view/burndata.h"
#include "src/view/burnimage.h"
#include "src/view/copyimage.h"
#include "src/view/modaldialog.h"
#include "src/view/execdialog.h"
#include "src/view/viewfilterfiles.h"
#include "src/view/filefilterrules.h"
#include "src/view/kybaboutdialog.h"
#include "src/frame/xatom-helper.h"
#include "src/frame/logger.h"
#include "src/frame/configuration.h"
#include "src/po/translation.h"
#include "src/model/logicmain.h"
#include "src/model/monitorcdrom.h"
#include "src/model/menufactory.h"
#include "src/data/datacdrom.h"

#include <pwd.h>
#include <unistd.h>

#include <fstream>

#include <QFileInfo>
#include <QMoveEvent>
#include <QFileSystemWatcher>

#define ICON_SIZE QSize(26, 26)

MainWindow::MainWindow(QStringList args, QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
    , m_oGroup(new QButtonGroup(this))
{
    QFont f; // 调整导航栏字体
    QPoint p;
    QUrl url;
    QFileInfo file;

    ui->setupUi(this);

    setWindowIcon(QIcon::fromTheme("disk-burner"));
    //setWindowTitle(_("Burner"));

    m_oCDROMS.clear();

    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(winId(), hints);

    // 日志记录打印
    connect(this, SIGNAL(info(QString)), LOGGER, SLOT(info(QString)));
    connect(this, SIGNAL(warn(QString)), LOGGER, SLOT(warn(QString)));
    connect(this, SIGNAL(error(QString)), LOGGER, SLOT(error(QString)));
    connect(this, SIGNAL(debug(QString)), LOGGER, SLOT(debug(QString)));
    LOGGER->registerName(this, _("MainWindow"));

    if (started())
    {
        error(_("Burner has started..."));
        EXEC->information(_("Starting"), _("Burner has been started,"
                                           "do not start again."), true);
        exit(0);
    }

    // 初始化业务界面
    connect(PAGE_BurnData, SIGNAL(busness(bool)), this, SLOT(slotsBusness(bool)));
    // connect(PAGE_BurnData, SIGNAL(busness(bool)), MENU, SIGNAL(setSetting(bool)));
    connect(PAGE_BurnData, SIGNAL(updateDisk(int,void*)), this, SLOT(slotsUpdateDisk(int,void*)));
    connect(PAGE_BurnImage, SIGNAL(busness(bool)), this, SLOT(slotsBusness(bool)));
    // connect(PAGE_BurnImage, SIGNAL(busness(bool)), MENU, SIGNAL(setSetting(bool)));
    connect(PAGE_CopyImage, SIGNAL(busness(bool)), this, SLOT(slotsBusness(bool)));
    // connect(PAGE_CopyImage, SIGNAL(busness(bool)), MENU, SIGNAL(setSetting(bool)));
    connect(PAGE_CopyImage, SIGNAL(busness(int)), this, SLOT(slotsChangeName(int)));
    ui->stackedWidget->addWidget(PAGE_BurnData);
    ui->stackedWidget->addWidget(PAGE_BurnImage);
    ui->stackedWidget->addWidget(PAGE_CopyImage);

    ui->navTitle->onlyShowLogo();
    ui->burnerTitle->onlyShowButton();
    // 自定义标题栏
    connect(this, SIGNAL(windowIconChanged(QIcon)), ui->navTitle, SLOT(onWindowIconChange(QIcon)));
    connect(this, SIGNAL(windowTitleChanged(QString)), ui->navTitle, SLOT(onWindowTitleChange(QString)));
    connect(this, SIGNAL(windowIconChanged(QIcon)), ui->burnerTitle, SLOT(onWindowIconChange(QIcon)));
    connect(this, SIGNAL(windowTitleChanged(QString)), ui->burnerTitle, SLOT(onWindowTitleChange(QString)));
    connect(ui->navTitle, SIGNAL(close()), this, SLOT(close()));
    connect(ui->navTitle, SIGNAL(min()), this, SLOT(showMinimized()));
    connect(ui->navTitle, &WindowTitle::max, [=](){
        if (Qt::WindowMaximized == windowState())
        {
            showNormal();
            ui->burnerTitle->onWindowShowMaxed(false);
        }
        else
        {
            showMaximized();
            ui->burnerTitle->onWindowShowMaxed(true);
        }
    });
    connect(ui->burnerTitle, SIGNAL(close()), this, SLOT(close()));
    connect(ui->burnerTitle, SIGNAL(min()), this, SLOT(showMinimized()));
    connect(ui->burnerTitle, &WindowTitle::max, [=](){
        if (Qt::WindowMaximized == windowState())
        {
            showNormal();
            ui->burnerTitle->onWindowShowMaxed(false);
        }
        else
        {
            showMaximized();
            ui->burnerTitle->onWindowShowMaxed(true);
        }
    });

    setWindowIcon(QIcon::fromTheme("disk-burner"));
    setWindowTitle(_("Burner"));
    ui->nav->setFocus();
    //setFocusPolicy(Qt::NoFocus);

    f = ui->btnBurnData->font();
    //f.setPointSize(15);
    f.setPixelSize(14);
    ui->btnBurnData->setFont(f);
    ui->btnBurnData->setText(_("BurnData"));
    ui->btnBurnData->setToolTip(_("BurnData"));
    ui->btnBurnData->setIconSize(ICON_SIZE);
    ui->btnBurnData->setIcon(QIcon(":/burnData.png"));
    connect(ui->btnBurnData, SIGNAL(clicked(bool)), this, SLOT(slotsBurnData()));

    ui->btnBurnImage->setFont(f);
    ui->btnBurnImage->setText(_("BurnImage"));
    ui->btnBurnImage->setToolTip(_("BurnImage"));
    ui->btnBurnImage->setIconSize(ICON_SIZE);
    ui->btnBurnImage->setIcon(QIcon(":/burnImage.png"));
    connect(ui->btnBurnImage, SIGNAL(clicked(bool)), this, SLOT(slotsBurnImage()));

    ui->btnCopyImage->setFont(f);
    ui->btnCopyImage->setText(_("CopyImage"));
    ui->btnCopyImage->setToolTip(_("CopyImage"));
    ui->btnCopyImage->setIconSize(ICON_SIZE);
    ui->btnCopyImage->setIcon(QIcon(":/copyImage.png"));
    connect(ui->btnCopyImage, SIGNAL(clicked(bool)), this, SLOT(slotsCopyImage()));

    f = ui->btnSetting->font();
    f.setPixelSize(14);
    ui->btnBusness->setFont(f);
    ui->btnSetting->setFont(f);
    ui->comboBoxBurner->setFont(f);
    ui->comboBoxBurner->clear();
    ui->comboBoxDisk->setFont(f);
    ui->comboBoxDisk->clear();
    f = ui->labelBurner->font();
    f.setWeight(60);
    f.setPixelSize(14);
    ui->labelBurner->setFont(f);
    ui->labelDisk->setFont(f);
    ui->labelBurner->setText(_("BurnerName:"));
    ui->labelDisk->setText(_("DiskName:"));
    //f = ui->btnBusness->font();
    //f.setPixelSize(18);
    //ui->btnBusness->setFont(f);
    info(_("Loaded over"));

    connect(ui->comboBoxBurner, SIGNAL(currentIndexChanged(int)), ui->comboBoxDisk, SLOT(setCurrentIndex(int)));
    connect(ui->comboBoxDisk, SIGNAL(currentIndexChanged(int)), ui->comboBoxBurner, SLOT(setCurrentIndex(int)));
    connect(ui->comboBoxDisk, SIGNAL(currentIndexChanged(int)), PAGE_BurnData, SIGNAL(changeCurrentIndex(int)));
    connect(this, SIGNAL(delData(int)), PAGE_BurnData, SIGNAL(delData(int)));
    connect(this, SIGNAL(delData(int)), VIEWFILTERFILE, SIGNAL(delNode(int)));
    connect(ui->comboBoxDisk, SIGNAL(currentIndexChanged(int)), this, SLOT(slotsAdjustBurnData(int)));

    ui->btnSetting->setCheckable(true);
    connect(ui->btnSetting, SIGNAL(toggled(bool)), this, SLOT(slotsBrowseOrSetting(bool)));
    connect(this, SIGNAL(browse(QString)), MAIN, SIGNAL(storagePath(QString)));

    connect(this, SIGNAL(busness(int)), MAIN, SLOT(slotsSetBusness(int)));
    connect(ui->btnBusness, SIGNAL(clicked(bool)), MAIN, SLOT(slotsDoBusness()));

    //connect(this, SIGNAL(position(QPoint)), MAIN, SIGNAL(position(QPoint)));
    connect(MAIN, SIGNAL(createISO(void*)), this, SLOT(slotsInitCreateISO(void*)));
    connect(MAIN, SIGNAL(noMedium(void*)), this, SLOT(slotsNoMediun(void*)));
    connect(MAIN, SIGNAL(deleteCDROM(void*)), this, SLOT(slotsRemoveCDROM(void*)));
    connect(MAIN, SIGNAL(emptyMedium(void*)), this, SLOT(slotsInitCreateISO(void*)));
    connect(MAIN, SIGNAL(appendableMedium(void*)), this, SLOT(slotsInitCreateISO(void*)));
    connect(MAIN, SIGNAL(finishedMedium(void*)), this, SLOT(slotsInitCreateISO(void*)));

    connect(MAIN, SIGNAL(appendableMedium(void*)), MENU, SLOT(newEarse(void*)));
    connect(MAIN, SIGNAL(finishedMedium(void*)), MENU, SLOT(newEarse(void*)));
    connect(MAIN, SIGNAL(noMedium(void*)), MENU, SLOT(newEarse(void*)));

    connect(MAIN, &LogicMain::iconChange, [=](QIcon icon){
        setWindowIcon(icon);
    });

    m_oGroup->addButton(ui->btnBurnData);
    m_oGroup->addButton(ui->btnBurnImage);
    m_oGroup->addButton(ui->btnCopyImage);
    if (args.size() == 1)
    {
        file.setFile(args.first());
        if (file.suffix() == "iso")
        {
            url.setUrl(args.first());
            PAGE_BurnImage->slotsNewISOPath(args.first());
            info(_("Need start with Burn Image, arg : %1").arg(args.first()));
            emit busness(LogicMain::BurnImage);
            ui->btnBurnImage->click();
        }
    }
    else
    {
        emit busness(LogicMain::CreateISO);
        emit ui->btnBurnData->click();
    }

    p.setX(pos().x() + width() / 2);
    p.setY(pos().y() + height() / 2);

    move(p);
    ABOUT->close();
    //emit position(p);
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    emit MAIN->exit();
    if (VIEWFILTERFILE)
    {
        VIEWFILTERFILE->close();
        VIEWFILTERFILE->destroy();
    }
    if (FILTERRULES)
    {
        FILTERRULES->close();
        FILTERRULES->destroy();
    }
    qApp->exit();
    QMainWindow::closeEvent(event);
}

void MainWindow::moveEvent(QMoveEvent *event)
{
    QPoint p;
    QPoint sp;

    p.setX(event->pos().x() + width() / 2);
    p.setY(event->pos().y() + height() / 2);

    ABOUT->updatePosition(p);
    MODAL->updatePosition(p);
    FILTERRULES->updatePosition(p);
    VIEWFILTERFILE->updatePosition(p);

    sp.setX(p.x() - EXEC->width() / 2);
    sp.setY(p.y() - EXEC->height() / 2);
    EXEC->move(sp);
    //emit position(sp);

    QMainWindow::moveEvent(event);
}

void MainWindow::paintEvent(QPaintEvent *event)
{
    QPalette pal = ui->burner->palette();
    if ("ukui-black" == MAIN->theme())
    {
        pal.setBrush(QPalette::Background, QColor("#484848"));
        ui->nav->setPalette(pal);
        pal.setBrush(QPalette::Background, QColor("#242424"));
        ui->burner->setPalette(pal);
    }
    else
    {
        pal.setBrush(QPalette::Background, QColor("#F0F0F0"));
        ui->nav->setPalette(pal);
        pal.setBrush(QPalette::Background, QColor("#FFFFFF"));
        ui->burner->setPalette(pal);
    }
    ui->comboBoxDisk->setToolTip(ui->comboBoxDisk->itemText(
                                     ui->comboBoxDisk->currentIndex()));
    QMainWindow::paintEvent(event);
}

void MainWindow::slotsBurnData()
{
    ui->burnData->show();
    ui->stackedWidget->setCurrentIndex(0);
    // 根据comboBoxDisk 下表结合其对应的光驱信息确认业务类型
    slotsAdjustBurnData(ui->comboBoxDisk->currentIndex());
}

void MainWindow::slotsBurnImage()
{
    ui->burnData->hide();
    ui->btnBusness->setText(_("BurnImage"));
    ui->stackedWidget->setCurrentIndex(1);
    emit busness(LogicMain::BurnImage);
    PAGE_BurnImage->update();
}

void MainWindow::slotsCopyImage()
{
    ui->burnData->hide();
    ui->btnBusness->setText(_("CopyImage"));
    ui->stackedWidget->setCurrentIndex(2);
    busness(LogicMain::GetISO);
    PAGE_CopyImage->check();
}

void MainWindow::slotsAdjustBurnData(int idx)
{
    DataCDROM *cdrom;

    cdrom = nullptr;
    if (idx < 0) return;

    if ( 0 != ui->stackedWidget->currentIndex() ) return;
    // 根据comboBoxDisk 下表结合其对应的光驱信息确认业务类型
    if (idx > 0)
    {
        if (idx >= m_oCDROMS.size()) return;
        cdrom = static_cast<DataCDROM *>(m_oCDROMS.at(idx));
        ui->btnSetting->setText(_("Setting"));
        switch (cdrom->mediumStatus())
        {
        case DataMedium::MEDIUM_EMPTY:
            ui->btnBusness->setText(_("BurnData"));
            emit busness(LogicMain::BurnData);
            break;
        case DataMedium::MEDIUM_INCOMPLETE:
            ui->btnBusness->setText(_("AppendBurn"));
            emit busness(LogicMain::AddBurn);
            break;
        default:
            ui->btnBusness->setText(_("DonotBurn"));
            emit busness(LogicMain::AddBurn);
            break;
        }
    }
    else
    {
        ui->btnSetting->setText(_("Browse"));
        ui->btnBusness->setText(_("CreateISO"));
        emit busness(LogicMain::CreateISO);
    }
    ui->comboBoxDisk->setToolTip(ui->comboBoxDisk->itemText(idx));
    PAGE_BurnData->refresh();
}

void MainWindow::slotsBusness(bool b)
{
    if (PAGE_BurnData == sender() && 0 == ui->stackedWidget->currentIndex())
    {
        ui->btnBusness->setEnabled(b);
        ui->btnSetting->setEnabled(b);
        MENU->setSetting(b);
    }
    else if (PAGE_BurnImage == sender() && 1 == ui->stackedWidget->currentIndex())
    {
        ui->btnBusness->setEnabled(b);
        MENU->setSetting(b);
    }
    else if (PAGE_CopyImage == sender() && 2 == ui->stackedWidget->currentIndex())
    {
        ui->btnBusness->setEnabled(b);
        MENU->setSetting(b);
    }
    else {}
}

void MainWindow::slotsInitCreateISO(void *d)
{
    int idx;
    DataCDROM *cdrom;

    cdrom = (DataCDROM *)d;
    if (-1 != m_oCDROMS.indexOf(d)) return;
    m_oCDROMS << d;

    ui->comboBoxBurner->addItem(cdrom->icon(), cdrom->display());
    ui->comboBoxDisk->addItem(cdrom->mediumIcon(), cdrom->mDisplay());

    idx = ui->comboBoxDisk->count() - 1;
    if (idx >= 0) ui->comboBoxDisk->setCurrentIndex(idx);
    else ui->comboBoxDisk->setCurrentIndex(0);
    PAGE_BurnData->refresh();

    if (0 != ui->stackedWidget->currentIndex()) return;
    switch (cdrom->mediumStatus())
    {
    case DataMedium::MEDIUM_EMPTY:
        ui->btnSetting->setText(_("Setting"));
        ui->btnBusness->setText(_("BurnData"));
        emit busness(LogicMain::BurnData);
        break;
    case DataMedium::MEDIUM_INCOMPLETE:
        ui->btnSetting->setText(_("Setting"));
        ui->btnBusness->setText(_("AppendBurn"));
        emit busness(LogicMain::AddBurn);
        break;
    default:
        if (cdrom->isCDROM())
        {
            ui->btnSetting->setText(_("Setting"));
            ui->btnBusness->setText(_("DonotBurn"));
            emit busness(LogicMain::AddBurn);
        }
        break;
    }
}

void MainWindow::slotsNoMediun(void * d)
{
    int idx;

    idx = m_oCDROMS.indexOf(d);
    if (-1 == idx) return;
    m_oCDROMS.takeAt(idx);
    delData(idx); // 先删除数据，后移除展示

    ui->comboBoxDisk->removeItem(idx);
    ui->comboBoxBurner->removeItem(idx);
    if (idx < ui->comboBoxDisk->count())
        ui->comboBoxDisk->setCurrentIndex(idx);
    else
        ui->comboBoxDisk->setCurrentIndex(0);
    debug(_("Delete item %1 of total %2").arg(idx).arg(ui->comboBoxBurner->count()));
}

void MainWindow::slotsRemoveCDROM(void *d)
{
    int idx;

    idx = m_oCDROMS.indexOf(d);
    if (-1 != idx)
    {
        debug(_("Remove CDROM from burn data."));
        m_oCDROMS.takeAt(idx);
    }
}
//#include <QDir>
void MainWindow::slotsBrowseOrSetting(bool b)
{
    QString path;
    QString testPath;
    QFileDialog fd;
    QList<QUrl> list = fd.sidebarUrls();
    int sidebarNum = 8;//最大添加U盘数，可以自己定义
    QString home = QDir::homePath().section("/", -1, -1);
    QString mnt = "/media/" + home + "/";
    QDir mntDir(mnt);
    mntDir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
    QFileInfoList filist = mntDir.entryInfoList();
    QList<QUrl> mntUrlList;
    for (int i = 0; i < sidebarNum && i < filist.size(); ++i) {
        QFileInfo fi = filist.at(i);
        if (fi.fileName() == "2691-6AB8") /* 990 9a0 */
                 continue;
        mntUrlList << QUrl("file://" + fi.filePath());
    }

    QFileSystemWatcher fsw(&fd);
    fsw.addPath("/media/" + home + "/");
    connect(&fsw, &QFileSystemWatcher::directoryChanged, &fd,
            [=, &sidebarNum, &mntUrlList, &list, &fd](const QString path) {
        QDir wmntDir(path);
        wmntDir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
        QFileInfoList wfilist = wmntDir.entryInfoList();
        mntUrlList.clear();
        for (int i = 0; i < sidebarNum && i < wfilist.size(); ++i) {
            QFileInfo fi = wfilist.at(i);
            if (fi.fileName() == "2691-6AB8") /* 990 9a0 */
                     continue;
            mntUrlList << QUrl("file://" + fi.filePath());
        }
        fd.setSidebarUrls(list + mntUrlList);
        fd.update();
    });

    connect(&fd, &QFileDialog::finished, &fd, [=, &list, &fd]() {
        fd.setSidebarUrls(list);
    });
    if (b)
    {
        if (0 == ui->comboBoxDisk->currentIndex())
        {
            //path = QFileDialog::getExistingDirectory(this, _("Browse"), path);
            //path = fd.getExistingDirectory(this, _("Browse"), path);
            //fd.setNameFilter(_("Forlder"));
            fd.setFileMode(QFileDialog::DirectoryOnly);
            fd.setSidebarUrls(list + mntUrlList);

            if (/*!path.isEmpty()*/ fd.exec() == QDialog::Accepted)
            {
                path = fd.selectedFiles().at(0);
                testPath = path + "/.t/.k";
                QDir dir(testPath);
                if (dir.mkpath(testPath))
                {
                    dir.removeRecursively();
                    browse(path);
                }
                else
                {
#if 0
                    QMessageBox::critical(this, _("Invalid path"),
                                          _("Cannot choose a read "
                                            "only path %1 to "
                                            "storage path").arg(path));
#endif
                    ui->btnSetting->setChecked(false);
                    return;
                }
            }
        }
        else
        {
            MENU->burnSetting(b);
        }
        ui->btnSetting->setChecked(false);
    }
}

void MainWindow::slotsUpdateDisk(int idx, void *d)
{
    DataCDROM *cdrom;

    cdrom = (DataCDROM *)d;

    ui->comboBoxDisk->setItemText(idx, cdrom->mDisplay());
    ui->comboBoxDisk->setItemIcon(idx, cdrom->mediumIcon());
}

void MainWindow::slotsCloseAll()
{
    close();
}

void MainWindow::slotsChangeName(int bus)
{
    if (ui->stackedWidget->currentWidget() != PAGE_CopyImage) return;
    switch (bus)
    {
    case LogicMain::GetISO:
        ui->btnBusness->setText(_("Get ISO"));
        busness(LogicMain::GetISO);
        break;
    default:
        ui->btnBusness->setText(_("Copy Disk"));
        busness(LogicMain::CopyData);
        break;
    }
}

bool MainWindow::started()
{
    QString comm;
    QDir dir("/proc/");
    QFileInfo file;
    ifstream in;
    std::string s;

    foreach (QFileInfo f, dir.entryInfoList())
    {
        if (f.baseName() == QString("%1").arg(getpid())
                || f.baseName() == "self" ||
                f.baseName() == "thread-self") continue;
        comm = QString("/proc/%1/comm").arg(f.baseName());
        file.setFile(comm);
        if (file.exists())
        {
            in.open(file.absoluteFilePath().toStdString());
            if (in.is_open())
            {
                getline(in, s);
                comm = QString::fromStdString(s);
                if (comm == "kylin-burner")
                    return true;
            }
            in.close();
        }
    }

    return false;
}

MainWindow::~MainWindow()
{
    MONITOR->stop();
    MAIN->destroy();
    MENU->destroy();
    PAGE_BurnImage->destroy();
    LOGGER->destroy();
    CFG->destroy();
    delete ui;
}

