#include "modaldialog.h"
#include "ui_modaldialog.h"
#include "src/frame/xatom-helper.h"
#include "src/frame/logger.h"
#include "src/po/translation.h"
#include "src/model/logicmain.h"

#include <QMoveEvent>
#include <QPushButton>
#include <QCoreApplication>

ModalDialog *ModalDialog::m_oInstance = nullptr;
QMutex       ModalDialog::m_oMutex;

ModalDialog::ModalDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ModalDialog)
{
    QFont f;
    ui->setupUi(this);
    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(winId(), hints);

    setAttribute(Qt::WA_ShowModal, true);

    ui->title->onlyShowInDialog();
    connect(ui->title, SIGNAL(close()), this, SLOT(slotsClose()));
    connect(this, SIGNAL(windowIconChanged(QIcon)), ui->title, SLOT(onWindowIconChange(QIcon)));
    connect(this, SIGNAL(windowTitleChanged(QString)), ui->title, SLOT(onWindowTitleChange(QString)));

    connect(this, SIGNAL(info(QString)), LOGGER, SLOT(info(QString)));
    connect(this, SIGNAL(warn(QString)), LOGGER, SLOT(warn(QString)));
    connect(this, SIGNAL(error(QString)), LOGGER, SLOT(error(QString)));
    connect(this, SIGNAL(debug(QString)), LOGGER, SLOT(debug(QString)));
    setWindowIcon(QIcon::fromTheme("disk-burner"));
    connect(ui->progress, SIGNAL(title(QString)), this, SLOT(slotsTitle(QString)));

    f = ui->btnBox->font();
    f.setPixelSize(14);
    ui->btnBox->setFont(f);
}

ModalDialog *ModalDialog::instance()
{
    if (nullptr == m_oInstance)
    {
        QMutexLocker locker(&m_oMutex);
        if (nullptr == m_oInstance)
        {
            m_oInstance = new ModalDialog();
        }
    }
    return m_oInstance;
}

QWidget *ModalDialog::showPage(int page)
{
    m_bCanClose = true;
    disconnect(this, SIGNAL(accepted()), ui->busnessSetting, SLOT(slotsAccept()));
    disconnect(this, SIGNAL(accepted()), ui->diskcheck, SLOT(accept()));
    //QCoreApplication::processEvents();弹窗界面递归导致闪退，所以去掉
    switch (page)
    {
    case Progress:
        setFixedSize(450, 230);
	ui->title->onlyVisibleClose(false);
        ui->btnBox->setStandardButtons(QDialogButtonBox::Cancel);
        ui->btnBox->button(QDialogButtonBox::Cancel)->setText(_("Cancel"));
        break;
    case Tip:
        setFixedSize(450, 230);
	ui->title->onlyVisibleClose(true);
        //ui->btnBox->setStandardButtons(QDialogButtonBox::NoButton);
        ui->btnBox->setStandardButtons(QDialogButtonBox::Ok);
        break;
    case BusnessSetting:
        setFixedSize(450, 530);
	ui->title->onlyVisibleClose(true);
        ui->btnBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
        ui->btnBox->button(QDialogButtonBox::Ok)->setText(_("OK"));
        ui->btnBox->button(QDialogButtonBox::Cancel)->setText(_("Cancel"));
        ui->btnBox->button(QDialogButtonBox::Ok)->setEnabled(false);
        connect(ui->busnessSetting, SIGNAL(title(QString)), this, SLOT(slotsTitle(QString)));
        connect(this, SIGNAL(accepted()), ui->busnessSetting, SLOT(slotsAccept()));
        connect(ui->busnessSetting, SIGNAL(changed(bool)), ui->btnBox->button(QDialogButtonBox::Ok), SLOT(setEnabled(bool)));
        break;
    case DataSetting:
        setFixedSize(450, 300);
	ui->title->onlyVisibleClose(true);
        ui->btnBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
        ui->btnBox->button(QDialogButtonBox::Ok)->setText(_("OK"));
        ui->btnBox->button(QDialogButtonBox::Cancel)->setText(_("Cancel"));
        connect(ui->dataSetting, SIGNAL(title(QString)), this, SLOT(slotsTitle(QString)));
        connect(this, SIGNAL(accepted()), ui->dataSetting, SLOT(accept()));
        connect(ui->dataSetting, SIGNAL(change(bool)), ui->btnBox->button(QDialogButtonBox::Ok), SLOT(setEnabled(bool)));
        ui->dataSetting->load();
        break;
    case DiskCheck:
        setFixedSize(450, 300);
	ui->title->onlyVisibleClose(true);
        slotsTitle(_("Disk Check"));
        ui->btnBox->setStandardButtons(QDialogButtonBox::Ok);
        ui->btnBox->button(QDialogButtonBox::Ok)->setText(_("DoDiskCheck"));
        connect(ui->diskcheck, SIGNAL(change(bool)), ui->btnBox->button(QDialogButtonBox::Ok), SLOT(setEnabled(bool)));
        connect(this, SIGNAL(accepted()), ui->diskcheck, SLOT(accept()));
        ui->diskcheck->change(ui->diskcheck->check());
        break;
    }
    ui->showPage->setCurrentIndex(page);
    return ui->showPage->currentWidget();
}

void ModalDialog::cancelEnable(bool b)
{
    m_bCanClose = false;
    ui->btnBox->button(QDialogButtonBox::Cancel)->setEnabled(b);
}

void ModalDialog::updatePosition(QPoint p)
{   
    m_oMainPos = p;
}

void ModalDialog::show()
{
    QPoint sp;
    switch (ui->showPage->currentIndex())
    {
    case Progress:
        //setFixedSize(450, 230);
        sp.setX(m_oMainPos.x()-225);
        sp.setY(m_oMainPos.y()-115);
        break;
    case Tip:
        //setFixedSize(450, 230);
        sp.setX(m_oMainPos.x()-225);
        sp.setY(m_oMainPos.y()-115);
        break;
    case BusnessSetting:
        //setFixedSize(450, 530);
        sp.setX(m_oMainPos.x()-225);
        sp.setY(m_oMainPos.y()-265);
        break;
    case DataSetting:
        //setFixedSize(450, 300);
        sp.setX(m_oMainPos.x()-225);
        sp.setY(m_oMainPos.y()-150);
        break;
    case DiskCheck:
        //setFixedSize(450, 300);
        sp.setX(m_oMainPos.x()-225);
        sp.setY(m_oMainPos.y()-150);
    }
    move(sp);
    QDialog::show();
}

void ModalDialog::paintEvent(QPaintEvent *event)
{
    QPalette pal = palette();

    //QCoreApplication::processEvents();弹窗界面递归导致栈资源用完闪退，所以去掉

    if ("ukui-black" == MAIN->theme())
        pal.setBrush(QPalette::Background, QColor("#484848"));
    else
        pal.setBrush(QPalette::Background, QColor("#FFFFFF"));
    setPalette(pal);
    //setWindowIcon(QIcon::fromTheme("burner"));
    QDialog::paintEvent(event);
}

void ModalDialog::closeEvent(QCloseEvent *event)
{
    slotsClose();
    QDialog::closeEvent(event);
}

void ModalDialog::slotsTitle(QString title)
{
    setWindowTitle(_("Burner") + " - " + title);
}

void ModalDialog::slotsClose()
{
    disconnect(this, SIGNAL(accepted()), ui->diskcheck, SLOT(accept()));
    if (!m_bCanClose) return;
    switch (ui->showPage->currentIndex())
    {
    case Progress:
        if (ui->btnBox->button(QDialogButtonBox::Cancel)->isEnabled())
        {
            reject();
        }
        break;
    case Tip:
        reject();
        break;
    case BusnessSetting:
        close();
        disconnect(this, SIGNAL(accepted()), ui->dataSetting, SLOT(accept()));
        break;
    case DataSetting:
        disconnect(ui->dataSetting, SIGNAL(title(QString)), this, SLOT(slotsTitle(QString)));
        disconnect(this, SIGNAL(accepted()), ui->dataSetting, SLOT(accept()));
        disconnect(ui->dataSetting, SIGNAL(change(bool)), ui->btnBox->button(QDialogButtonBox::Ok), SLOT(setEnabled(bool)));
        close();
        break;
    case DiskCheck:
        disconnect(ui->diskcheck, SIGNAL(change(bool)), ui->btnBox->button(QDialogButtonBox::Ok), SLOT(setEnabled(bool)));
        disconnect(this, SIGNAL(accepted()), ui->diskcheck, SLOT(accept()));
        close();
        break;
    }
}

ModalDialog::~ModalDialog()
{
    delete ui;
}
